"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DingzDaBaseAccessory = void 0;
const dingzLogHelper_1 = require("./dingzLogHelper");
const axios_1 = __importDefault(require("axios"));
const axios_retry_1 = __importDefault(require("axios-retry"));
const settings_1 = require("../settings");
const errors_1 = require("./errors");
const accessoryEventBus_1 = require("./accessoryEventBus");
const axiosDebugHelper_1 = require("./axiosDebugHelper");
const chalk_1 = __importDefault(require("chalk"));
/**
 * DingzDaBase Accessory is our base class
 * - getDeviceStateUpdate() is the one method every child class should override
 * TODO: #140 [FIX] use interface & class inheritance to make this clearer
 */
class DingzDaBaseAccessory {
    constructor(platform, accessory) {
        var _a;
        this.platform = platform;
        this.accessory = accessory;
        this.reachabilityState = null;
        this.eb = new accessoryEventBus_1.AccessoryEventBus();
        /**
         * Handler for request errors
         * @param e AxiosError: the error returned by this.request()
         */
        this.handleRequestErrors = (e) => {
            var _a, _b;
            if (e && e.isAxiosError) {
                this.reachabilityState = new Error();
                switch (e.code) {
                    case 'ECONNABORTED':
                        this.log.error('HTTP ECONNABORTED Connection aborted --> ' + this.device.address);
                        break;
                    case 'EHOSTDOWN':
                        this.log.error('HTTP EHOSTDOWN Host down --> ' + this.device.address);
                        break;
                    default:
                        this.log.error(`HTTP ${e.code} ${e.message} ${(_b = (_a = e.response) === null || _a === void 0 ? void 0 : _a.statusText) !== null && _b !== void 0 ? _b : ' '}--> ` +
                            this.device.address);
                        break;
                }
            }
            else if (e instanceof errors_1.DeviceNotReachableError) {
                this.log.error(`DeviceNotReachableError --> ${this.accessory.displayName} (${this.device.address})`);
                this.reachabilityState = new Error();
            }
            else {
                this.log.error(e.message + '\n' + e.stack);
                throw new Error(`Device request failed -> escalating error: ${e.message}`);
            }
        };
        // Set-up axios instances
        this.device = this.accessory.context.device;
        this.baseUrl = `http://${this.device.address}`;
        this.log = new dingzLogHelper_1.DingzLogger(this.accessory.context.device.name, platform.log);
        // Set the update interval in seconds
        const updateInterval = !this.platform.config.pollerInterval ||
            this.platform.config.pollerInterval < settings_1.STATE_UPDATE_INTERVAL
            ? settings_1.STATE_UPDATE_INTERVAL
            : this.platform.config.pollerInterval;
        this.request = axios_1.default.create({
            baseURL: this.baseUrl,
            // We set the retry timeout at 0.7 times the update interval
            // This is 3.5s minimumtimeout for the minimum interval of 5s
            timeout: updateInterval * 1000 * 0.7,
            headers: { Token: (_a = this.device.token) !== null && _a !== void 0 ? _a : '' },
        });
        /**
         * Set-up the protected and static axios instance
         *
         * This leads to up to 18s delay between retries before failing
         * Each request will time-out after 3000 ms
         * maxdelay -> 2^7*100 ms -> 12.8s
         */
        (0, axios_retry_1.default)(this.request, DingzDaBaseAccessory.axiosRetryConfig);
        (0, axios_retry_1.default)(axios_1.default, DingzDaBaseAccessory.axiosRetryConfig);
        this.debugHelper = new axiosDebugHelper_1.AxiosDebugHelper(this.request, this.log);
        // .. and finally set-up the interval that triggers updates or
        // changes the reachability state of the device
        this.platform.eb.on("requestStateUpdate" /* PlatformEvent.REQUEST_STATE_UPDATE */, () => {
            this.getDeviceStateUpdate()
                .then(() => {
                // The device update was successful
                if (this.reachabilityState !== null) {
                    // Update reachability -- obviously, we're online again
                    this.reachabilityState = null;
                    this.log.info(chalk_1.default.green('[ALIVE]'), `Device --> recovered from unreachable state (${this.device.address})`);
                }
            })
                .catch((e) => {
                this.log.warn(chalk_1.default.red('[DEAD]'), `Device --> entered unreachable state (${this.device.address})`);
                this.handleRequestErrors(e);
            });
        });
        // Register listener for updated device info (e.g. on restore with new IP)
        this.platform.eb.on("updateDeviceInfo" /* PlatformEvent.UPDATE_DEVICE_INFO */, (deviceInfo) => {
            var _a;
            const device = this.accessory.context.device;
            if (deviceInfo.mac === device.mac) {
                this.log.debug('Updated device info received -> update accessory address');
                // Update core info (mainly address, maybe token too)
                if (device.address !== deviceInfo.address ||
                    device.token !== deviceInfo.token) {
                    this.log.info('Accessory IP changed for', this.accessory.displayName, '-> Updating accessory from ->', device.address, 'to', deviceInfo.address);
                    this.accessory.displayName = device.name;
                    device.address = deviceInfo.address;
                    device.token = deviceInfo.token;
                    this.baseUrl = `http://${this.device.address}`;
                    // Set specific default headers
                    this.request.defaults.baseURL = this.baseUrl;
                    this.request.defaults.timeout = settings_1.RETRY_TIMEOUT * 1000;
                    this.request.defaults.headers.common = {
                        Token: (_a = device.token) !== null && _a !== void 0 ? _a : '',
                    };
                    // update AccessoryInformation
                    this.accessory.context.device = deviceInfo;
                    this.reconfigureAccessory();
                }
                // Set accessory to reachable and
                this.reachabilityState = null;
            }
        });
    }
    // Override these if specific actions needed on updates on restore
    reconfigureAccessory(init = false) {
        this.log.debug(`reconfigureAccessory(${init}) not implemented for`, this.device.accessoryClass);
    }
    getDeviceStateUpdate() {
        this.log.warn('getDeviceStateUpdate() not implemented for', this.device.accessoryClass);
        return Promise.reject(new errors_1.MethodNotImplementedError(this.device.accessoryClass));
    }
    /*
     * This method is optional to implement. It is called when HomeKit ask to identify the accessory.
     * Typical this only ever happens at the pairing process.
     */
    identify() {
        var _a;
        this.log.info('Identify! -> Who am I? I am a', (_a = this.device.accessoryClass) !== null && _a !== void 0 ? _a : 'unkown device type', 'listening to the name of', this.device.name, 'a.k.a.', this.accessory.displayName, '-> MAC:', this.device.mac);
    }
}
exports.DingzDaBaseAccessory = DingzDaBaseAccessory;
DingzDaBaseAccessory.axiosRetryConfig = {
    retries: settings_1.REQUEST_RETRIES,
    retryDelay: axios_retry_1.default.exponentialDelay,
    shouldResetTimeout: true,
};
DingzDaBaseAccessory.axios = axios_1.default;
//# sourceMappingURL=dingzDaBaseAccessory.js.map