"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MyStromButtonAccessory = void 0;
const cockatiel_1 = require("cockatiel");
const commonTypes_1 = require("./lib/commonTypes");
const dingzTypes_1 = require("./lib/dingzTypes");
const dingzDaBaseAccessory_1 = require("./lib/dingzDaBaseAccessory");
// Policy for long running tasks, retry every hour
const retrySlow = cockatiel_1.Policy.handleAll()
    .orWhenResult((retry) => retry === true)
    .retry()
    .exponential({ initialDelay: 10000, maxDelay: 60 * 60 * 1000 });
/**
 * Platform Accessory
 * An instance of this class is created for each accessory your platform registers
 * Each accessory may expose multiple services of different service types.
 */
class MyStromButtonAccessory extends dingzDaBaseAccessory_1.DingzDaBaseAccessory {
    constructor(_platform, _accessory) {
        var _a;
        super(_platform, _accessory);
        this._platform = _platform;
        this._accessory = _accessory;
        // Eventually replaced by:
        this.buttonState = commonTypes_1.ButtonAction.SINGLE_PRESS;
        this.switchButtonState = dingzTypes_1.ButtonState.OFF;
        this.batteryLevel = 0;
        this.chargingState = false;
        // Set Base URL
        this.log.debug('Setting informationService Characteristics ->', this.device.model);
        this.accessory
            .getService(this.platform.Service.AccessoryInformation)
            .setCharacteristic(this.platform.Characteristic.Manufacturer, 'MyStrom AG')
            .setCharacteristic(this.platform.Characteristic.AppMatchingIdentifier, 'ch.mystrom.iOSApp')
            .setCharacteristic(this.platform.Characteristic.Model, this.device.model)
            .setCharacteristic(this.platform.Characteristic.FirmwareRevision, 'N/A')
            .setCharacteristic(this.platform.Characteristic.HardwareRevision, 'PQWBB1')
            .setCharacteristic(this.platform.Characteristic.SerialNumber, this.device.mac);
        const batteryService = (_a = this.accessory.getService(this.platform.Service.Battery)) !== null && _a !== void 0 ? _a : this.accessory.addService(this.platform.Service.Battery);
        batteryService
            .getCharacteristic(this.platform.Characteristic.BatteryLevel)
            .on("get" /* CharacteristicEventTypes.GET */, this.getBatteryLevel.bind(this));
        batteryService
            .getCharacteristic(this.platform.Characteristic.StatusLowBattery)
            .on("get" /* CharacteristicEventTypes.GET */, this.getStatusBatteryLow.bind(this));
        batteryService
            .getCharacteristic(this.platform.Characteristic.ChargingState)
            .on("get" /* CharacteristicEventTypes.GET */, this.getChargingState.bind(this));
        // get the StatelessProgrammableSwitch service if it exists,
        // otherwise create a new StatelessProgrammableSwitch service
        // you can create multiple services for each accessory
        this.log.info('Create Stateless Programmable Switch');
        this.buttonServiceSetup();
        // Set the callback URL (Override!)
        retrySlow.execute(() => {
            this.platform.setButtonCallbackUrl({
                baseUrl: this.baseUrl,
                token: this.device.token,
                endpoints: ['generic'], // Buttons need the 'generic' endpoint specifically set
            });
        });
    }
    // Set-up the button (override in subclasses)
    buttonServiceSetup() {
        var _a, _b;
        const buttonService = (_a = this.accessory.getService(this.platform.Service.StatelessProgrammableSwitch)) !== null && _a !== void 0 ? _a : this.accessory.addService(this.platform.Service.StatelessProgrammableSwitch);
        buttonService.setCharacteristic(this.platform.Characteristic.Name, (_b = this.device.name) !== null && _b !== void 0 ? _b : `${this.accessory.context.device.model}`);
        buttonService
            .getCharacteristic(this.platform.Characteristic.ProgrammableSwitchEvent)
            .on("get" /* CharacteristicEventTypes.GET */, this.getButtonState.bind(this));
        // Stateful Programmable Switches are not anymore exposed in HomeKit. However,
        //  the "ProgrammableSwitchOutputState" Characteristic added to a
        // StatelessProgrammableSwitch (i.e., a button), can be read out -- and used --
        // by third-party apps for HomeKite, allowing users to create automations
        // not only based on the button events, but also based on a state that's toggled
        buttonService
            .getCharacteristic(this.platform.Characteristic.ProgrammableSwitchOutputState)
            .on("get" /* CharacteristicEventTypes.GET */, this.getButtonState.bind(this));
        this.platform.eb.on("deviceAction" /* PlatformEvent.ACTION */, (mac, action, battery) => {
            if (mac === this.device.mac) {
                this.buttonState = action !== null && action !== void 0 ? action : commonTypes_1.ButtonAction.SINGLE_PRESS;
                this.batteryLevel = battery;
                const batteryService = this.accessory.getService(this.platform.Service.Battery);
                if (batteryService) {
                    batteryService
                        .getCharacteristic(this.platform.Characteristic.BatteryLevel)
                        .updateValue(battery);
                }
                const buttonService = this.accessory.getService(this.platform.Service.StatelessProgrammableSwitch);
                const ProgrammableSwitchEvent = this.platform.Characteristic.ProgrammableSwitchEvent;
                if (buttonService) {
                    this.log.debug(`Button (${this.device.mac}) pressed -> ${action}`);
                    switch (action) {
                        case commonTypes_1.ButtonAction.SINGLE_PRESS:
                            buttonService
                                .getCharacteristic(ProgrammableSwitchEvent)
                                .updateValue(ProgrammableSwitchEvent.SINGLE_PRESS);
                            break;
                        case commonTypes_1.ButtonAction.DOUBLE_PRESS:
                            buttonService
                                .getCharacteristic(ProgrammableSwitchEvent)
                                .updateValue(ProgrammableSwitchEvent.DOUBLE_PRESS);
                            break;
                        case commonTypes_1.ButtonAction.LONG_PRESS:
                            buttonService
                                .getCharacteristic(ProgrammableSwitchEvent)
                                .updateValue(ProgrammableSwitchEvent.LONG_PRESS);
                            break;
                        default:
                            this.log.info('Heartbeat (Unknown action) ->', this.device.address, '-> Device is alive, though, will try to update info!');
                            this.platform
                                .getMyStromDeviceInfo({
                                address: this.device.address,
                                token: this.device.token,
                            })
                                .then((data) => {
                                var _a;
                                if (typeof data !== 'undefined') {
                                    const info = data;
                                    this.accessory.context.device.hwInfo = info;
                                    if (batteryService) {
                                        batteryService
                                            .getCharacteristic(this.platform.Characteristic.ChargingState)
                                            .updateValue((_a = info.charge) !== null && _a !== void 0 ? _a : false);
                                    }
                                }
                            });
                            break;
                    }
                }
            }
        });
    }
    getButtonState(callback) {
        const currentState = this.buttonState;
        callback(this.reachabilityState, currentState);
    }
    getSwitchButtonState(callback) {
        const currentState = this.switchButtonState;
        this.log.info('Get Switch State ->', currentState);
        callback(this.reachabilityState, currentState);
    }
    getBatteryLevel(callback) {
        const currentLevel = this.batteryLevel;
        callback(this.reachabilityState, currentLevel);
    }
    getStatusBatteryLow(callback) {
        var _a;
        const currentLevel = (_a = this.batteryLevel) !== null && _a !== void 0 ? _a : 0;
        callback(null, currentLevel <= 10
            ? this.platform.Characteristic.StatusLowBattery.BATTERY_LEVEL_LOW
            : this.platform.Characteristic.StatusLowBattery.BATTERY_LEVEL_NORMAL);
    }
    getChargingState(callback) {
        const currentState = this.chargingState;
        callback(this.reachabilityState, currentState);
    }
    // Buttons can not be queried -- always resolve Promise
    getDeviceStateUpdate() {
        this.log.debug('getDeviceStateUpdate() not implemented for', this.device.accessoryClass);
        return Promise.resolve();
    }
}
exports.MyStromButtonAccessory = MyStromButtonAccessory;
//# sourceMappingURL=myStromButtonAccessory.js.map