"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MyStromPIRAccessory = void 0;
const async_mutex_1 = require("async-mutex");
const commonTypes_1 = require("./lib/commonTypes");
const errors_1 = require("./lib/errors");
const dingzDaBaseAccessory_1 = require("./lib/dingzDaBaseAccessory");
const chalk_1 = __importDefault(require("chalk"));
const limit_number_1 = __importDefault(require("limit-number"));
/**
 * Platform Accessory
 * An instance of this class is created for each accessory your platform registers
 * Each accessory may expose multiple services of different service types.
 */
class MyStromPIRAccessory extends dingzDaBaseAccessory_1.DingzDaBaseAccessory {
    constructor(_platform, _accessory) {
        var _a, _b, _c, _d, _e;
        super(_platform, _accessory);
        this._platform = _platform;
        this._accessory = _accessory;
        this.mutex = new async_mutex_1.Mutex();
        this.pirState = {
            motion: false,
            temperature: 0,
            light: 0,
        };
        // Set Base URL
        this.mystromDeviceInfo = this.device.hwInfo;
        this.log.debug('Setting informationService Characteristics ->', this.device.model);
        this.accessory
            .getService(this.platform.Service.AccessoryInformation)
            .setCharacteristic(this.platform.Characteristic.Manufacturer, 'myStrom AG')
            .setCharacteristic(this.platform.Characteristic.AppMatchingIdentifier, 'ch.mystrom.iOSApp')
            .setCharacteristic(this.platform.Characteristic.Model, this.device.model)
            .setCharacteristic(this.platform.Characteristic.FirmwareRevision, (_a = this.mystromDeviceInfo.version) !== null && _a !== void 0 ? _a : 'N/A')
            .setCharacteristic(this.platform.Characteristic.HardwareRevision, 'PQWBB1')
            .setCharacteristic(this.platform.Characteristic.SerialNumber, this.device.mac);
        // get the LightBulb service if it exists, otherwise create a new LightBulb service
        // you can create multiple services for each accessory
        this.log.info('Create Motion Sensor');
        this.temperatureService =
            (_b = this.accessory.getService(this.platform.Service.TemperatureSensor)) !== null && _b !== void 0 ? _b : this.accessory.addService(this.platform.Service.TemperatureSensor);
        this.temperatureService.setCharacteristic(this.platform.Characteristic.Name, `Temperature ${this.device.name}`);
        // create handlers for required characteristics
        this.temperatureService
            .getCharacteristic(this.platform.Characteristic.CurrentTemperature)
            .on("get" /* CharacteristicEventTypes.GET */, this.getTemperature.bind(this));
        // Add the LightSensor that's integrated in the dingz
        // API: /api/v1/light
        this.lightService =
            (_c = this.accessory.getService(this.platform.Service.LightSensor)) !== null && _c !== void 0 ? _c : this.accessory.addService(this.platform.Service.LightSensor);
        this.lightService.setCharacteristic(this.platform.Characteristic.Name, `Light ${this.device.name}`);
        // create handlers for required characteristics
        this.lightService
            .getCharacteristic(this.platform.Characteristic.CurrentAmbientLightLevel)
            .on("get" /* CharacteristicEventTypes.GET */, this.getLightLevel.bind(this));
        this.motionService =
            (_d = this.accessory.getService(this.platform.Service.MotionSensor)) !== null && _d !== void 0 ? _d : this.accessory.addService(this.platform.Service.MotionSensor);
        this.motionService.setCharacteristic(this.platform.Characteristic.Name, `Motion ${this.device.name}`);
        if (!((_e = this.platform.config.motionPoller) !== null && _e !== void 0 ? _e : true)) {
            // Implement *push* event handling
            this.platform.eb.on("deviceAction" /* PlatformEvent.ACTION */, (mac, action) => {
                this.log.debug(`Processing DingzEvent.ACTION ${action}`);
                if (mac === this.device.mac) {
                    this.log.debug(`Motion detected -> ${action}`);
                    let isMotion;
                    switch (action) {
                        case commonTypes_1.ButtonAction.PIR_MOTION_STOP:
                            isMotion = false;
                            break;
                        case commonTypes_1.ButtonAction.PIR_MOTION_START:
                        default:
                            isMotion = true;
                            break;
                    }
                    this.log.debug('Motion Update from PUSH');
                    this.pirState.motion = isMotion;
                    this.motionService.setCharacteristic(this.platform.Characteristic.MotionDetected, this.pirState.motion);
                }
            });
        }
    }
    setCallbackUrl() {
        this.getButtonCallbackUrl()
            .then((callBackUrl) => {
            var _a;
            if (this.platform.config.callbackOverride) {
                this.log.warn('Override callback URL ->', callBackUrl);
                // Set the callback URL (Override!)
                this.platform.setButtonCallbackUrl({
                    baseUrl: this.baseUrl,
                    token: this.device.token,
                    endpoints: ['pir/generic'],
                });
            }
            else if (
            // FIXME: Needed because of #511
            !((_a = callBackUrl === null || callBackUrl === void 0 ? void 0 : callBackUrl.url) === null || _a === void 0 ? void 0 : _a.includes(this.platform.getCallbackUrl()))) {
                this.log.warn('Update existing callback URL ->', callBackUrl);
                // Set the callback URL (Override!)
                this.platform.setButtonCallbackUrl({
                    baseUrl: this.baseUrl,
                    token: this.device.token,
                    oldUrl: callBackUrl.url,
                    endpoints: ['pir/generic'],
                });
            }
            else {
                this.log.debug('Callback URL already set ->', callBackUrl === null || callBackUrl === void 0 ? void 0 : callBackUrl.url);
            }
        })
            .catch(this.handleRequestErrors.bind(this));
    }
    /**
     * Returns the callback URL for the device
     */
    async getButtonCallbackUrl() {
        const getCallbackEndpoint = '/api/v1/action/pir/generic';
        this.log.debug('Getting the callback URL -> ', getCallbackEndpoint);
        return await this.request.get(getCallbackEndpoint).then((response) => {
            return response.data;
        });
    }
    // Get updated device info and update the corresponding values
    getDeviceStateUpdate() {
        return this.getDeviceReport()
            .then((report) => {
            var _a, _b, _c;
            if (report) {
                if (this.reachabilityState !== null) {
                    // Update reachability -- obviously, we're online again
                    this.reachabilityState = null;
                    this.log.info(chalk_1.default.green('[ALIVE]'), `Device --> recovered from unreachable state (${this.device.address})`);
                    this.setCallbackUrl();
                }
                // If we are in motion polling mode, update motion from poller
                if ((_a = this.platform.config.motionPoller) !== null && _a !== void 0 ? _a : true) {
                    this.log.info('Motion POLLING of', this.device.name, 'enabled');
                    const isMotion = report.motion;
                    // Only update if motionService exists *and* if there's a change in motion'
                    if (this.pirState.motion !== isMotion) {
                        this.log.debug('Motion Update from POLLER');
                        this.pirState.motion = isMotion;
                        this.motionService
                            .getCharacteristic(this.platform.Characteristic.MotionDetected)
                            .updateValue(this.pirState.motion);
                    }
                }
                // Update temperature and light in any case
                this.pirState.temperature = (0, limit_number_1.default)(-273.5, 100, (_b = report.temperature) !== null && _b !== void 0 ? _b : 0); // fixes #300
                this.temperatureService
                    .getCharacteristic(this.platform.Characteristic.CurrentTemperature)
                    .updateValue(this.pirState.temperature);
                this.pirState.light = (0, limit_number_1.default)(0.0001, 100000, (_c = report.light) !== null && _c !== void 0 ? _c : 0); // fixes #300
                this.lightService
                    .getCharacteristic(this.platform.Characteristic.CurrentAmbientLightLevel)
                    .updateValue(this.pirState.light);
                return Promise.resolve();
            }
            else {
                return Promise.reject(new errors_1.DeviceNotReachableError());
            }
        })
            .catch((e) => {
            return Promise.reject(e);
        });
    }
    /**
     * Handle Handle the "GET" requests from HomeKit
     * to get the current value of the "Ambient Light Level" characteristic
     */
    getLightLevel(callback) {
        var _a, _b;
        const light = (0, limit_number_1.default)(0.0001, 100000, (_b = (_a = this.pirState) === null || _a === void 0 ? void 0 : _a.light) !== null && _b !== void 0 ? _b : 0); // fixes #300
        this.log.debug('Get Characteristic Ambient Light Level ->', light, ' lux');
        callback(this.reachabilityState, light);
    }
    /**
     * Handle Handle the "GET" requests from HomeKit
     * to get the current value of the "Temperature" characteristic
     */
    getTemperature(callback) {
        var _a, _b;
        const temperature = (0, limit_number_1.default)(-273.5, 100, (_b = (_a = this.pirState) === null || _a === void 0 ? void 0 : _a.temperature) !== null && _b !== void 0 ? _b : 0); // fixes #300
        this.log.debug('Get Characteristic Temperature ->', temperature, '° C');
        callback(this.reachabilityState, temperature);
    }
    /**
     * Handle Handle the "GET" requests from HomeKit
     * to get the current value of the "Motion Detected" characteristic
     */
    getMotionDetected(callback) {
        // set this to a valid value for MotionDetected
        const isMotion = this.pirState.motion;
        callback(this.reachabilityState, isMotion);
    }
    async getDeviceReport() {
        const getSensorsUrl = `${this.baseUrl}/api/v1/sensors`;
        return await this.request.get(getSensorsUrl).then((response) => {
            return response.data;
        });
    }
    /*
     * This method is optional to implement. It is called when HomeKit ask to identify the accessory.
     * Typical this only ever happens at the pairing process.
     */
    identify() {
        this.log.debug('Identify! -> Who am I? I am', this.accessory.displayName);
    }
}
exports.MyStromPIRAccessory = MyStromPIRAccessory;
//# sourceMappingURL=myStromPIRAccessory.js.map